@@ -11,6 +11,7 @@ from pandas import json_normalize
 from nexus_api.NexusRequest import NexusRequest
 from nexus_api.NexusValue import NexusValue
 
+
 def WarningsAndJson(func):
     """Decorator including InsecureRequestWarning and then JSON() format"""
 
@@ -42,6 +43,7 @@ def no_row_limit_decorator(fnc):
         else:
             response = fnc(NX, df_nexus)
         return response
+
     return wrapper
 
 
@@ -146,7 +148,7 @@ class APINexus:
         url_completa = self.url_NX + "/api/Alarms/alarm/" + uid
         return requests.get(url_completa, verify=False, headers=header)
 
-    #@WarningsAndJson -- no se necesita ya que se requiere comparar el objeto response
+    # @WarningsAndJson -- no se necesita ya que se requiere comparar el objeto response
     def callPostAckAlarm(self, uid, status: str):
         """Lectura de alarmas compartidas con el mismo token
         status: 'ARE' or 'EXR'
@@ -155,12 +157,12 @@ class APINexus:
             header = self.header.copy()
             header['nexusapiversion'] = '2.0'
             url_completa = self.url_NX + "/api/Alarms/alarm/" + uid + "/acknowledge"
-            body = "\""+status+"\""
+            body = "\"" + status + "\""
             return requests.post(url_completa, verify=False, headers=header, data=body)
         else:
             raise ValueError("Status must be 'ARE' or 'EXR'")
 
-    #@WarningsAndJson
+    # @WarningsAndJson
     def callPostAlarmEvent(self, uid, msg: str):
         """
         Usado para insertar mensajes en el histórico de la alarma con uid específico
@@ -172,7 +174,6 @@ class APINexus:
         url_completa = self.url_NX + "/api/Alarms/alarm/" + uid + "/event"
         return requests.post(url_completa, verify=False, headers=header, json={'Message': msg})
 
-
     def callPostTagInsert(self, variable_name):
         """consulta de tags con permisos de escritura"""
         url_get = self.url_NX + "/api/Tags/writable"
@@ -296,7 +297,7 @@ class APINexus:
             df2.drop(columns=["name"], inplace=True)
             if df2['timeStamp'].dtype == 'datetime64[ns]':
                 print('timeStamp in datetime format: please check that it is in UTC')
-                df2['timeStamp'] = df2['timeStamp'].astype('int64')/1e9
+                df2['timeStamp'] = df2['timeStamp'].astype('int64') / 1e9
             print(str(df2))
             payload = pandas.DataFrame.to_json(
                 df2, date_format="epoch", orient="records"
@@ -361,7 +362,7 @@ class APINexus:
             df2.drop(columns=["name"], inplace=True)
             if df2['timeStamp'].dtype == 'datetime64[ns]':
                 print('timeStamp in datetime format: please check that it is in UTC')
-                df2['timeStamp'] = df2['timeStamp'].astype('int64')/1e9
+                df2['timeStamp'] = df2['timeStamp'].astype('int64') / 1e9
             print(str(df2))
             payload = pandas.DataFrame.to_json(
                 df2, date_format="epoch", orient="records"
@@ -440,7 +441,7 @@ class APINexus:
 
         return dataReceived
 
-    def filter_installation(self, Datefrom, Dateto, columnas, filter_txt, resolucion=3, fuente=0):
+    def filter_installation(self, Datefrom, Dateto, columnas, filter_txt=None, resolucion=3, fuente=0):
         """
         la funcion recibe como parametros la fecha ini, fecha fin, un df con los uid y
         los nombres de las variables de la instalación y el filtro de texto aplicar
@@ -449,7 +450,7 @@ class APINexus:
             Datefrom (datetime)
             Dateto (datetime)
             columnas (dataframe)
-            filter_txt (list or str)
+            filter_txt (optonal: list or str)
             resolucion (int): de 0 a 10 [ RES_30_SEC, RES_1_MIN, RES_15_MIN, RES_1_HOUR, RES_1_DAY, RES_1_MONTH, RES_1_YEAR, RES_5_MIN, RES_200_MIL, RES_500_MIL, RES_1_SEC ]
             fuente (int): [0 -->RAW, 1 -->STATS_PER_HOUR, 2 -->STATS_PER_DAY, 3 -->STATS_PER_MONTH, 4 -->TRANSIENT]
         Returns:
@@ -469,20 +470,25 @@ class APINexus:
         fecha_ini = Datefrom
         fecha_fin = Dateto
         nexusRequest = NexusRequest(uids, fecha_ini, fecha_fin, fuente, resolucion)
-
         filtered_hist = self.callGetTagsHistory(nexusRequest)
         filtered_hist = json_normalize(filtered_hist)
+
+        # Check that dataframe is not empty
+        if filtered_hist.empty:
+            raise VoidDataframeException
+        # Check content
+        if not {'timeStamp', 'uid', 'value'}.issubset(filtered_hist.columns):
+            raise CorruptDataframeException(filtered_hist)
+
         filtered_hist.timeStamp = pandas.to_datetime(filtered_hist.timeStamp, unit='s')
-        for item in uids:
-            name_uid = list(columnas[columnas['uid'] == item].name)[0]
-            filtered_hist['uid'] = filtered_hist['uid'].replace(item, name_uid)
+        diccio = dict([(i, j) for i, j in zip(columnas.uid, columnas.name)])
+        filtered_hist['name'] = filtered_hist['uid'].map(diccio)
         filtered_hist = filtered_hist.rename(columns={'uid': 'name'})
         filtered_hist.set_index(filtered_hist.timeStamp, inplace=True)
         del filtered_hist['timeStamp']
-        print('Data retrieved')
-        return (filtered_hist)
+        return filtered_hist
 
-    def filter_tagview(self, Datefrom, Dateto, columnas, uid_vista, filter_txt, resolucion=3, fuente=0):
+    def filter_tagview(self, Datefrom, Dateto, columnas, uid_vista, filter_txt=None, resolucion=3, fuente=0):
         """
         La funcion recibe como parametros la fecha ini, fecha fin, un df con los uid y
         los nombres de las variables de la instalación y el filtro de texto aplicar [pueden ser varios]
@@ -490,7 +496,7 @@ class APINexus:
             Datefrom (datetime)
             Dateto (datetime)
             columnas (dataframe)
-            filter_txt (list or str)
+            filter_txt (optional: list or str)
             resolucion (int): de 0 a 10 [ RES_30_SEC, RES_1_MIN, RES_15_MIN, RES_1_HOUR, RES_1_DAY, RES_1_MONTH, RES_1_YEAR, RES_5_MIN, RES_200_MIL, RES_500_MIL, RES_1_SEC ]
             fuente (int): [0 -->RAW, 1 -->STATS_PER_HOUR, 2 -->STATS_PER_DAY, 3 -->STATS_PER_MONTH, 4 -->TRANSIENT]
         Returns:
@@ -517,11 +523,16 @@ class APINexus:
         filtered_hist = self.callGetDataviewHistory(uid_vista, nexusRequest)
         filtered_hist = json_normalize(filtered_hist)
 
+        # Check that dataframe is not empty
+        if filtered_hist.empty:
+            raise VoidDataframeException
+        # Check content
+        if not {'timeStamp', 'uid', 'value'}.issubset(filtered_hist.columns):
+            raise CorruptDataframeException(filtered_hist)
         filtered_hist.timeStamp = pandas.to_datetime(filtered_hist.timeStamp, unit='s')
         diccio = dict([(i, j) for i, j in zip(columnas.uid, columnas.name)])
         filtered_hist['name'] = filtered_hist['uid'].map(diccio)
-        print('Data retrieved')
-        return (filtered_hist)
+        return filtered_hist
 
     def get_alarms_uids_by_names(self, names):
         alarms = json_normalize(self.callGetAlarms())
@@ -536,3 +547,18 @@ class APINexus:
         for group in groups:
             uids.extend(alarms['uid'][alarms['group'].str.contains(group, case=False)].to_list())
         return uids
+
+
+class VoidDataframeException(Exception):
+    def __init__(self, message='dataframe is empty'):
+        self.message = message
+        # Show full traceback
+        super().__init__(self.message)
+
+
+class CorruptDataframeException(Exception):
+    def __init__(self, df, message='dataframe does not match Nexus structure'):
+        self.message = f'{message} - {df.head()}'
+        self.df = df
+        # Show full traceback
+        super().__init__(self.message)
